// generic character head script

// TODO: fix double includes properly, added the following line temporarily to fix include issues
#include "PaletteSwap.as"
#include "PixelOffsets.as"
#include "RunnerTextures.as"

const s32 NUM_HEADFRAMES = 4;
const s32 NUM_UNIQUEHEADS = 30;
const int FRAMES_WIDTH = 8 * NUM_HEADFRAMES;
const string sauron_path = "../Mods/Skin_Heads/Heads/";
const int sauron_size = 1024;

//handling DLCs

class HeadsDLC {
	string filename;
	bool do_teamcolour;
	bool do_skincolour;

	HeadsDLC(string file, bool team, bool skin) {
		filename = file;
		do_teamcolour = team;
		do_skincolour = skin;
	}
};

const array<HeadsDLC> dlcs = {
	//vanilla
	HeadsDLC("Entities/Characters/Sprites/Heads.png", true, true),
	//flags of the world
	HeadsDLC("Entities/Characters/Sprites/Heads2.png", false, false)
};

const int dlcs_count = dlcs.length;

int get_dlc_number(int headIndex)
{
	if (headIndex > 255) {
		if ((headIndex % 256) > NUM_UNIQUEHEADS) {
			return Maths::Min(dlcs_count - 1, Maths::Floor(headIndex / 255.0f));
		}
	}
	return 0;
}

int getHeadFrame(CBlob@ blob, int headIndex, bool default_pack)
{
	if(headIndex < NUM_UNIQUEHEADS)
	{
		return headIndex * NUM_HEADFRAMES;
	}

	//special heads logic for default heads pack
	if(default_pack && (headIndex == 255 || headIndex == NUM_UNIQUEHEADS))
	{
		CRules@ rules = getRules();
		bool holidayhead = false;
		if(rules !is null && rules.exists("holiday"))
		{
			const string HOLIDAY = rules.get_string("holiday");
			if(HOLIDAY == "Halloween")
			{
				headIndex = NUM_UNIQUEHEADS + 43;
				holidayhead = true;
			}
			else if(HOLIDAY == "Christmas")
			{
				headIndex = NUM_UNIQUEHEADS + 61;
				holidayhead = true;
			}
		}

		//if nothing special set
		if(!holidayhead)
		{
			string config = blob.getConfig();
			if(config == "undeadbuilder")
			{
				headIndex = NUM_UNIQUEHEADS;
			}
			else if(config == "undeadknight")
			{
				headIndex = NUM_UNIQUEHEADS + 1;
			}
			else if(config == "undeadarcher")
			{
				headIndex = NUM_UNIQUEHEADS + 44;
			}
			else if(config == "undeadnecromancer")
			{
				headIndex = NUM_UNIQUEHEADS + 49;
			}	
			else if(config == "undeadslayer")
			{
				headIndex = NUM_UNIQUEHEADS + 18;
			}	
			else if(config == "undeadstalker")
			{
				headIndex = NUM_UNIQUEHEADS + 33;
			}
			else if(config == "undeadarsonist")
			{
				headIndex = NUM_UNIQUEHEADS + 37;
			}
			else if(config == "undeadmystic")
			{
				headIndex = NUM_UNIQUEHEADS + 40;
			}				
			else if(config == "migrant")
			{
				Random _r(blob.getNetworkID());
				headIndex = 69 + _r.NextRanged(2); //head scarf or old
			}
			else
			{
				// default
				headIndex = NUM_UNIQUEHEADS;
			}
		}
	}

	return (((headIndex - NUM_UNIQUEHEADS / 2) * 2) +
	        (blob.getSexNum() == 0 ? 0 : 1)) * NUM_HEADFRAMES;
}

string getHeadTexture(int headIndex)
{
	return dlcs[get_dlc_number(headIndex)].filename;
}

void onPlayerInfoChanged(CSprite@ this)
{
	LoadHead(this, this.getBlob().getHeadNum());
}

CSpriteLayer@ LoadHead(CSprite@ this, int headIndex)
{
	this.RemoveSpriteLayer("head");
	// add head
	int dlc_number = get_dlc_number(headIndex);
	HeadsDLC dlc = dlcs[dlc_number];
	CSpriteLayer@ head = this.addSpriteLayer("head", dlc.filename, 16, 16,
	                     (dlc.do_teamcolour ? this.getBlob().getTeamNum() : 0),
	                     (dlc.do_skincolour ? this.getBlob().getSkinNum() : 0));
	CBlob@ blob = this.getBlob();

	// set defaults
	headIndex = headIndex % 256; // DLC heads
	s32 headFrame = getHeadFrame(blob, headIndex, dlc_number == 0);

	blob.set_s32("head index", headFrame);
	if (head !is null)
	{
		Animation@ anim = head.addAnimation("default", 0, false);
		anim.AddFrame(headFrame);
		anim.AddFrame(headFrame + 1);
		anim.AddFrame(headFrame + 2);
		head.SetAnimation(anim);

		head.SetFacingLeft(blob.isFacingLeft());
	}
	return head;
}

void onGib(CSprite@ this)
{
	if (g_kidssafe)
	{
		return;
	}

	CBlob@ blob = this.getBlob();
	if (blob !is null && blob.getName() != "bed")
	{
		int frame = blob.get_s32("head index");
		int framex = frame % FRAMES_WIDTH;
		int framey = frame / FRAMES_WIDTH;

		Vec2f pos = blob.getPosition();
		Vec2f vel = blob.getVelocity();
		f32 hp = Maths::Min(Maths::Abs(blob.getHealth()), 2.0f) + 1.5;
		makeGibParticle(getHeadTexture(blob.getHeadNum()),
		                pos, vel + getRandomVelocity(90, hp , 30),
		                framex, framey, Vec2f(16, 16),
		                2.0f, 20, "/BodyGibFall", blob.getTeamNum());
	}
}

void onTick(CSprite@ this)
{
	CBlob@ blob = this.getBlob();

	ScriptData@ script = this.getCurrentScript();
	if (script is null)
		return;

	if (blob.getShape().isStatic())
	{
		script.tickFrequency = 60;
	}
	else
	{
		script.tickFrequency = 1;
	}


	// head animations
	CSpriteLayer@ head = this.getSpriteLayer("head");

	// load head when player is set or it is AI
	if (head is null && (blob.getPlayer() !is null || (blob.getBrain() !is null && blob.getBrain().isActive()) || blob.getTickSinceCreated() > 3))
	{
		@head = LoadHead(this, blob.getHeadNum());
	}

	if (head !is null)
	{
		Vec2f offset;

		// pixeloffset from script
		// set the head offset and Z value according to the pink/yellow pixels
		int layer = 0;
		Vec2f head_offset = getHeadOffset(blob, -1, layer);

		// behind, in front or not drawn
		if (layer == 0)
		{
			head.SetVisible(false);
		}
		else
		{
			head.SetVisible(this.isVisible());
			head.SetRelativeZ(layer * 0.25f);
		}

		offset = head_offset;

		// set the proper offset
		Vec2f headoffset(this.getFrameWidth() / 2, -this.getFrameHeight() / 2);
		headoffset += this.getOffset();
		headoffset += Vec2f(-offset.x, offset.y);
		headoffset += Vec2f(0, -2);
		head.SetOffset(headoffset);

		if (blob.hasTag("dead") || blob.hasTag("dead head"))
		{
			head.animation.frame = 2;

			// sparkle blood if cut throat
			if (getNet().isClient() && getGameTime() % 2 == 0 && blob.hasTag("cutthroat"))
			{
				Vec2f vel = getRandomVelocity(90.0f, 1.3f * 0.1f * XORRandom(40), 2.0f);
				ParticleBlood(blob.getPosition() + Vec2f(this.isFacingLeft() ? headoffset.x : -headoffset.x, headoffset.y), vel, SColor(255, 126, 0, 0));
				if (XORRandom(100) == 0)
					blob.Untag("cutthroat");
			}
		}
		else if (blob.hasTag("attack head"))
		{
			head.animation.frame = 1;
		}
		else
		{
			head.animation.frame = 0;
		}
	}
}
